/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */
 
/*
 * $Header: /cvs/glassfish/appserv-api/src/java/com/sun/appserv/management/util/misc/PrintMillis.java,v 1.1 2006/12/02 06:04:32 llc Exp $
 * $Revision: 1.1 $
 * $Date: 2006/12/02 06:04:32 $
 */
package com.sun.appserv.management.util.misc;

/**
	Useful for printing elapsed timings out. Example:
	
	<code>
	final PrintMillis	timer	= new PrintMillis();
	...
	timer.println( "start" );
	...
	timer.println( "middle" );
	...
	...
	timer.println( "end" );
	</code>
 */
public class PrintMillis
{
	private long	mLast	= System.currentTimeMillis();
	
		public
	PrintMillis()
	{
		mLast	= System.currentTimeMillis();
	}
	
	/**
		Print out the milliseconds that have elapsed since the last call.
	 */
		public void
	println( String msg )
	{
		final long	elapsed	= System.currentTimeMillis() - mLast;
		
		// this printing to System.out is BY DESIGN, so leave it.
		System.out.println( (msg == null ? "" : msg) + ": " + elapsed );
		
		mLast	= System.currentTimeMillis();
	}
}
